from time import time

from karabo.bound import KARABO_CLASSINFO
from PipeToZeroMQ import PipeToZeroMQ, conversion, device_schema

from . import shmem_utils
from ._version import version as deviceVersion


@KARABO_CLASSINFO("ShmemToZMQ", deviceVersion)
class ShmemToZMQ(PipeToZeroMQ.PipeToZeroMQ):
    def initialization(self):
        super().initialization()
        self._shmem_handler = shmem_utils.ShmemCircularBufferReceiver()

    def onInput(self, input_channel):
        actual = self.getActualTimestamp()
        input_tic = time()
        self.info["inputUpdated"] += 1
        self.info["dataRecv"] = input_channel.size()
        all_meta = input_channel.getMetaData()

        data = {}
        meta = {}

        for idx in range(input_channel.size()):
            # Read metadata
            metadata = self._extract_metadata(all_meta, idx)
            source = metadata["source"]
            if source not in self.sources:
                self.appendSchema(device_schema.timestamp_schema(source))
                self.sources.add(source)
            self._time_info(metadata, actual, self.info)

            # Read data
            hash_data = input_channel.read(idx)

            # filters
            if self.allowed_sources and source not in self.allowed_sources:
                continue
            forward, ignore = self._filter_properties(hash_data, source)

            meta[source] = conversion.meta_to_dict(metadata)
            meta[source]["ignored_keys"] = ignore
            # only this bit differs from PipeToZeroMQ.onInput
            dic, arr = conversion.hash_to_dict(
                hash_data, paths=forward, version=self.version
            )
            for shmem_handle_path in dic.pop("calngShmemPaths", []):
                shmem_handle = dic.pop(shmem_handle_path, None)
                if shmem_handle_path is None:
                    self.log.INFO(
                        f"Hash from {source} did not have {shmem_handle_path}"
                    )
                    continue
                elif shmem_handle_path == "":
                    self.log.INFO(
                        f"Hash from {source} had empty {shmem_handle_path}"
                    )
                    continue
                actual_data = self._shmem_handler.get(shmem_handle)
                arr[shmem_handle_path] = actual_data

            data[source] = (dic, arr)

        # forward data to all connected ZMQ sockets
        self._send(data, meta)

        output_tic = time()
        self.info["onInputTotal"] = 1000 * (output_tic - input_tic)

        # update properties on device
        self._updateProperties(output_tic)
        # block if device is in passive state
        self.monitoring.wait()
